import argparse

from . import package

base_url = "https://pypi.org"


def cli():
    parser = argparse.ArgumentParser(
        prog="pipmore",
        description="Find the latest package version supported by a specific python version",
        epilog="https://github.com/andregri/pipmore",
    )

    parser.add_argument("package_name")
    parser.add_argument("py_version")

    args = parser.parse_args()

    max_release = None
    releases = package.releases(base_url, args.package_name)

    for release in releases:
        pkg = package.Release(base_url, args.package_name, release)
        requires_python = pkg.requires_python()
        max_release = package.find_max_release(
            release, max_release, args.py_version, requires_python
        )

    print(max_release)


if __name__ == "__main__":
    cli()



def from_classifiers(classifiers):
    """
    Return a list of python versions extracted from classifiers.
    Return an empty list if classifiers are not defined.
    """

    py_versions = []
    for cl in classifiers:
        version = cl.split(" :: ")[-1]
        if version in ["3.4", "3.5", "3.6", "3.7", "3.8", "3.9", "3.10", "3.11"]:
            py_versions.append(version)

    return py_versions


def minimum_from_pyversions(py_versions):
    """
    Return the minimum python version from a list of versions
    """
    minors = [int(v.split(".")[-1]) for v in py_versions]
    index = minors.index(min(minors))

    return py_versions[index]


def minimum_from_classifiers(classifiers):
    """
    Returns the minimum python version listed in package classifiers
    """
    py_versions = from_classifiers(classifiers)
    return minimum_from_pyversions(py_versions)


def minimum_from_requires(requires_python):
    """
    Parse the minimum python version from requires_python info
    """
    if requires_python[:2] == ">=":
        return requires_python[2:]


def minimum_from_combined(requires_python, classifiers):
    """
    Returns the minimum python version combining the classifiers
    and the requires_python field
    """

    min_requires = minimum_from_requires(requires_python)
    min_classifier = minimum_from_classifiers(classifiers)

    return minimum_from_pyversions([min_requires, min_classifier])



import unittest

from pipmore import pyversion


class Test_PyVersionFromClassifier(unittest.TestCase):
    def test_pyversions_from_classifier(self):
        classifiers = ["Programming Language :: Python :: 3.10"]
        expected = ["3.10"]
        actual = pyversion.from_classifiers(classifiers)
        self.assertEqual(actual, expected)

    def test_empty_classifiers(self):
        classifiers = []
        expected = []
        actual = pyversion.from_classifiers(classifiers)
        self.assertEqual(actual, expected)

    def test_minimum_from_classifiers(self):
        classifiers = [
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.10",
        ]
        expected = "3.4"
        actual = pyversion.minimum_from_classifiers(classifiers)
        self.assertEqual(actual, expected)

    def test_minimum_from_requires_ge(self):
        requires = ">=3.8"
        expected = "3.8"
        actual = pyversion.minimum_from_requires(requires)
        self.assertEqual(actual, expected)

    def test_classifier_smaller_than_requires(self):
        classifiers = [
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.10",
        ]
        requires = ">=3.8"
        expected = "3.4"
        actual = pyversion.minimum_from_combined(requires, classifiers)
        self.assertEqual(actual, expected)

    def test_requires_smaller_than_classifier(self):
        classifiers = ["Programming Language :: Python :: 3.10"]
        requires = ">=3.8"
        expected = "3.8"
        actual = pyversion.minimum_from_combined(requires, classifiers)
        self.assertEqual(actual, expected)


if __name__ == "__main__":
    unittest.main()



import requests

from . import package, pyversion

headers = {"Accept": "application/json"}


def releases(index_url, package_name):
    """
    Return a list of releases for a package.
    """

    releases = []

    r = requests.get(f"{index_url}/pypi/{package_name}/json", headers=headers)
    if r.status_code == 200:
        r_json = r.json()
        releases = list(r_json["releases"].keys())

    return releases


def find_max_release(release, max_release, py_version, requires_python=None):
    """
    Return the max release that satifies the requires_python constraint.
    If requires_python is None, don't use py_version.
    """
    if not max_release:
        return release

    if requires_python:
        min_requires_python = pyversion.minimum_from_requires(requires_python)
        if py_version >= min_requires_python:
            return max(max_release, release)
    else:
        return max(max_release, release)

    return max_release


def get_info(index_url, package_name, release):
    """
    Returns info dictionary from a package release
    """

    r = requests.get(f"{index_url}/pypi/{package_name}/{release}/json", headers=headers)
    if r.status_code == 200:
        r_json = r.json()
        return r_json.get("info")

    raise RuntimeError("couldn't get release info")


class Release:
    """
    Release is a specific release of a python package
    """

    def __init__(self, index_url, package_name, release):
        self.index_url = index_url
        self.package_name = package_name
        self.release = release
        self.info = get_info(index_url, package_name, release)

    def requires_python(self):
        """
        Return the requires_python of a package release,
        otherwise an empty string
        """
        return self.info.get("requires_python")

    def classifiers(self):
        """
        Return the classifiers of a package release
        """
        return self.info.get("classifiers")



import unittest

from pipmore import package


class Test_MaxRelease(unittest.TestCase):
    def test_max_release_with_requires_python_not_satisfied(self):
        release = "4.5.6"
        max_release = "4.3.1"
        requires_python = ">=3.8"
        py_version = "3.6"

        expected = "4.3.1"
        actual = package.find_max_release(
            release, max_release, py_version, requires_python
        )

        self.assertEqual(actual, expected)

    def test_max_release_with_requires_python_satisfied(self):
        release = "4.5.6"
        max_release = "4.3.1"
        requires_python = ">=3.8"
        py_version = "3.9"

        expected = "4.5.6"
        actual = package.find_max_release(
            release, max_release, py_version, requires_python
        )

        self.assertEqual(actual, expected)

    def test_max_release_without_requires_python(self):
        release = "4.2.8"
        max_release = "4.3.1"
        py_version = "3.6"

        expected = "4.3.1"
        actual = package.find_max_release(release, max_release, py_version)

        self.assertEqual(actual, expected)

    def test_max_release_without_requires_python_and_max_release_is_newest(self):
        release = "4.3.1"
        max_release = "4.2.8"
        py_version = "3.6"

        expected = "4.3.1"
        actual = package.find_max_release(release, max_release, py_version)

        self.assertEqual(actual, expected)



from setuptools import setup

setup(
    name="pipmore",
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={"console_scripts": ["pipmore=pipmore.cli:cli"]},
)
